from time import sleep
from selenium.webdriver.common.keys import Keys
from .tool import error_log
import traceback

def add_to_group(self, contact_name: str):
    """Add a new participant to the group

    Args:
        contact_name (str): The contact name of who you want to add
    """

    try:

        self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span').click()

        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div'
            )

        except:
            print('You are not a group admin!')
            
        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div:nth-child(5) > div:nth-child(2) > div.TbtXF'
        ).click()

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > span > div > div > div > div > div > div > div:nth-child(2) > div > label > div > div._2_1wd.copyable-text.selectable-text'
        ).send_keys(contact_name)
        
        sleep(0.5)
        
        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > span > div > div > div > div > div > div > div:nth-child(2) > div > label > div > div._2_1wd.copyable-text.selectable-text'
        ).send_keys(Keys.ENTER)
            
        sleep(0.5)

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > span > div > div > div > div > div > div > span._3IGMG > div > div > div'
        ).click()

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > span > div > div > div > div > div > div._1uJw_ > div._1dwBj._3xWLK'
        ).click()

    except:
        error_log(traceback.format_exc())
    
    try:
        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button'
        ).click()
    except:
        pass
    

def remove_from_group(self, participant_name: str):
    """Removes a participant from the group

    Args:
        participant_name (str): The contact name or number of who you want to remove
    """

    try:

        self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span').click()

        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div'
            )

        except:
            print('You are not a group admin!')
            return

        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div:nth-child(5) > div.-ZdaK > div > div > div._3TVPy'
        ).click()

        sleep(0.5)

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > div > div > div > div > div > div > div:nth-child(2) > div > label > div > div._2_1wd.copyable-text.selectable-text'
        ).send_keys(participant_name)

        sleep(0.5)
        
        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > div > div > div > div > div > div > div._1C2Q3._36Jt6 > div:nth-child(1) > div > div > div > div > div'
        ).click()

        sleep(0.5)

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(4) > div > ul > li:nth-child(2)'
        ).click()

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > div > div > div > div > div > div > header > div > div._215wZ > button'
        ).click()
    
    except:
        error_log(traceback.format_exc())
    
    try:
        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button'
        ).click()
    except:
        pass


def make_group_admin(self, participant_name: str):
    """Makes someone a group admin

    Args:
        participant_name (str): [The contact name or number of who you want to make admin
    """

    try:

        self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span'
        ).click()

        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div'
            )

        except:
            print('You are not a group admin!')
            return

        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div:nth-child(5) > div.-ZdaK > div > div > div._3TVPy'
        ).click()

        sleep(0.5)

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > div > div > div > div > div > div > div:nth-child(2) > div > label > div > div._2_1wd.copyable-text.selectable-text'
        ).send_keys(participant_name)

        sleep(0.5)
        
        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > div > div > div > div > div > div > div._1C2Q3._36Jt6 > div:nth-child(1) > div > div > div > div > div'
        ).click()

        sleep(0.5)

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(4) > div > ul > li:nth-child(1)'
        ).click()

        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(2) > div > div > div > div > div > div > div > header > div > div._215wZ > button'
        ).click()
    
    except:
        error_log(traceback.format_exc())
    
    try:
        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button'
        ).click()
    except:
        pass


def select_chat_by_name(self, chat_name: str):
    """Go to the selected chat by its number

    Args:
        chat_name (str): Contact/Group name
    """

    self.driver.find_element_by_css_selector(
        '#side > div.SgIJV > div > label > div > div._2_1wd.copyable-text.selectable-text'
    ).send_keys(chat_name)

    self.driver.find_element_by_css_selector(
        '#side > div.SgIJV > div > label > div > div._2_1wd.copyable-text.selectable-text'
    ).send_keys(Keys.ENTER)


def select_chat_by_number(self, chat_number: int):
    """Go to the selected chat by its number

    Args:
        chat_number (int): Contact number. Example: 1NPAXXXXXXX
    """

    self.driver.get(f'https://web.whatsapp.com/send?phone={chat_number}')
    sleep(5)


import re
import shutil
from os import path
from time import sleep
from send2trash import send2trash
from selenium.webdriver.common.keys import Keys
from .tool import error_log
import traceback

last = ''

class Message:

    def __init__(self, author, content, time, date):
        self.author  = author
        self.time    = time
        self.date    = date
        self.content = content
    
    def __str__(self):
        return f'Author: {self.author}\nTime: {self.time}\nDate: {self.date}\nContent: {self.content}'


def last_message(self):
    """Gets the last message from the chat
    
    Returns:
        Class: Last message
    """

    global Message

    try:
        info = self.driver.execute_script('''
            var a = document.querySelectorAll(".message-in");
            return a[a.length - 1].querySelector('.copyable-text').dataset.prePlainText;
        ''')
            
        content = self.driver.execute_script('''
            var a = document.querySelectorAll(".message-in");
            return a[a.length - 1].querySelector('.copyable-text').innerText;
        ''')

        time = re.compile(r'(\d+:\d+(\s)?(AM|PM)?)').findall(info)[0][0]
        date = re.compile(r'(\d+/\d+/\d+)').findall(info)[0]
        author = re.compile(r'] (.*):').findall(info)[0]

        return Message(author=author, content=content, time=time, date=date)
    except:
        error_log(traceback.format_exc())


def new_message(self):
    """Returns True for a new message

    Returns:
        Bool: New message
    """

    global last
        
    try:
        message = self.last_message()
        if last == '': last = message.content

        if message.content != last:
            last = message.content
            return True
            
        else:
            return False
    except:
        pass


def send(self, message: str):
    """Sends a message

    Args:
        message (str): The message you want to send
    """

    try:
        chat = self.driver.find_element_by_css_selector(
            '#main > footer > div.vR1LG._3wXwX.copyable-area > div._2A8P4 > div > div._2_1wd.copyable-text.selectable-text')

        if message.find('\n'):
            for line in message.split('\n'):
                chat.send_keys(line)
                chat.send_keys(Keys.SHIFT + Keys.ENTER)
            chat.send_keys(Keys.ENTER)
        else:
            chat.send_keys(message)
    except:
        error_log(traceback.format_exc())

    
def reply(self, message: str):
    """Replies to the last message

    Args:
        message (str): The message you want to send
    """
        
    self.driver.execute_script('''
        var a = document.querySelectorAll('.message-in');
        var elem = a[a.length -1]
        var clickEvent = document.createEvent('MouseEvents');
        clickEvent.initEvent('dblclick', true, true);
        elem.dispatchEvent(clickEvent);
    ''')
    self.send(message)


def reply_privately(self, message: str):
    """Sends a message message privatly to the last message in chat

    Args:
        message (str): The message you want to send
    """

    try:
        group_name = self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span'
        ).text

        self.driver.execute_script('''
            var event = new MouseEvent('mouseover', {
                'view': window,
                'bubbles': true,
                'cancelable': true
            });

            var a = document.querySelectorAll('.message-in > div');
            var element = a[a.length -1];
            
            element.dispatchEvent(event);
        ''')

        self.driver.find_element_by_css_selector(
            '.message-in > div > div > span > div > div'
        ).click()
        
        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(4) > div > ul > li:nth-child(2)'
        ).click()

        self.send(message)
        self.select_chat(group_name)

    except:
        error_log(traceback.format_exc())


def reply_file_privately(self, file_path: str):
    """Sends a file privatly to the last message in chat

    Args:
        file_path (str, absolute path): The file of the path you want to send
    """

    try:
        group_name = self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span'
        ).text

        self.driver.execute_script('''
            var event = new MouseEvent('mouseover', {
                'view': window,
                'bubbles': true,
                'cancelable': true
            });

            var a = document.querySelectorAll('.message-in > div');
            var element = a[a.length -1];
            
            element.dispatchEvent(event);
        ''')

        self.driver.find_element_by_css_selector(
            '.message-in > div > div > span > div > div'
        ).click()
        
        self.driver.find_element_by_css_selector(
            '#app > div > span:nth-child(4) > div > ul > li:nth-child(2)'
        ).click()

        self.send_file(file_path)
        self.select_chat(group_name)

    except:
        error_log(traceback.format_exc())


def send_file(self, file_path: str):
    """Sends a file

    Args:
        file_path (str, absolute path): The file of the path you want to send
    """

    regex = re.compile(r'(\w+\.(\w+))')
    file_name = file_path.split('\\')[-1]
    isZip = False

    if regex.findall(file_name):
        if regex.findall(file_name)[0] in ['png', 'jpg', 'mp4', '3gpp']:
            type = 1
        else:
            type = 3

    else:
        shutil.make_archive(file_name, 'zip', file_name)

        file_path = path.abspath(file_name + '.zip')
        isZip = True

        type = 3

    self.driver.execute_script('''
        document.querySelector("#main > footer > div.vR1LG._3wXwX.copyable-area > div.EBaI7._23e-h > div._2C9f1 > div > div").click()
    ''')

    sleep(0.7)

    img_box = self.driver.find_element_by_css_selector(
        f'#main > footer > div.vR1LG._3wXwX.copyable-area > div.EBaI7._23e-h > div._2C9f1 > div > span > div > div > ul > li:nth-child({type}) > button > input[type=file]'
    )
        
    img_box.send_keys(file_path)

    while True:
        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._1sFTb > span > div > span > div > div > div._36Jt6.tEF8N > span > div > div > span'
            ).click()

            break
        except:
            pass

    if isZip:
        send2trash(file_name + '.zip')


from .tool import error_log
import traceback

def get_recent_chats(self):
    """Returns a list of all recent chats

    Returns:
        List: All the recent chats
    """
    
    try:
        array = self.driver.execute_script('''
        let asd = document.querySelectorAll('._2Z4DV');
        let newArray = new Array();
        for (var i = 0; i < asd.length; i++){
            newArray.push(asd[i].querySelector('.N2dUK').textContent);
        };

        return newArray;
        ''')

        return array
    except:
        error_log(traceback.format_exc())


def get_pinned_chats(self):
    """Returns a list of all pinned chats

    Returns:
        List: All the pinned chats
    """

    try:
        array = self.driver.execute_script('''
        let pin = document.querySelectorAll('._3EhXO');
        let newArray = new Array();
        for (var i = 0; i < pin.length; i++) {
            newArray.push(pin[i].parentNode.parentNode.parentNode.parentNode.querySelector('._3Dr46').textContent)
        }

        return newArray
        ''')

        return array
    except:
        error_log(traceback.format_exc())


def get_group_invite_link(self):
    """Returns group invite link

    Returns:
        str: Group invite link
    """

    try:

        self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span').click()

        sleep(1)

        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div')

        except:
            print('You are not a group admin!')
            return

        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div:nth-child(5) > div:nth-child(3)').click()
        
        sleep(1)

        group_link = self.driver.find_element_by_css_selector('#group-invite-link-anchor').text

        self.driver.find_element_by_css_selector('#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button').click()
        
        sleep(1)

        try:
            self.driver.find_element_by_css_selector('#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button').click()
        except:
            pass

        return group_link
    
    except:
        error_log(traceback.format_exc())


from time import sleep
from selenium.webdriver.common.keys import Keys
from .tool import error_log
import traceback

def change_group_description(self, description: str):
    """Changes the group description

    Args:
        description (str): New group description
    """

    try:

        self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span').click()

        sleep(1)

        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div')

        except:
            print('You are not a group admin!')
            return

        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div').click()

        description_dom = self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div._3rhi1 > div > div._2_1wd.copyable-text.selectable-text')

        description_dom.clear()

        if description.find('\n'):
            for line in description.split('\n'):
                description_dom.send_keys(line)
                description_dom.send_keys(Keys.SHIFT + Keys.ENTER)
            description_dom.send_keys(Keys.ENTER)

        else:
            description_dom.send_keys(description)
            
    except:
        error_log(traceback.format_exc())
    
    try:
        self.driver.find_element_by_css_selector('#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button').click()
    except:
        pass


def change_group_name(self, name: str):
    """Changes the group name

    Args:
        name (str): New group name
    """

    try:

        self.driver.find_element_by_css_selector(
            '#main > header > div._2uaUb > div.z4t2k > div > span').click()

        sleep(1)

        try:
            self.driver.find_element_by_css_selector(
                '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt._82zXh > div._3NATg > div > div > span._2zDdK > div')

        except:
            print('You are not a group admin!')
            return

        self.driver.find_element_by_class_name('_1JAUF').click()

        self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt.bRenh > div._2O6GW._3Ss_B._1lemF._3Ihuv > div._3rhi1.e1K_H._1nQew > span._2zDdK > div').click()

        group_name_dom = self.driver.find_element_by_css_selector(
            '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div._3ZEdX._3hiFt.bRenh > div._2O6GW._3Ss_B._1lemF._1fB8E._3Ihuv > div._3rhi1 > div > div._2_1wd.copyable-text.selectable-text')

        group_name_dom.clear()
        group_name_dom.send_keys(name + Keys.ENTER)

    except:
        error_log(traceback.format_exc())
    
    try:
        self.driver.find_element_by_css_selector('#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > header > div > div._215wZ > button').click()
    except:
        pass


def leave_group(self):
    """Leaves the group you are"""

    self.driver.find_element_by_css_selector(
        '#main > header > div._2uaUb > div.z4t2k > div > span').click()

    self.driver.find_element_by_css_selector(
        '#app > div > div > div.Akuo4 > div._1Flk2._3xysY > span > div > span > div > div > section > div:nth-child(6) > div').click()

    self.driver.find_element_by_css_selector(
        '#app > div > span:nth-child(2) > div > div > div > div > div > div > div._1uJw_ > div._1dwBj._3xWLK').click()


from .chat import (
    last_message, new_message, send, reply, 
    send_file, reply_privately, reply_file_privately
)
from .actions import (
    add_to_group, remove_from_group, make_group_admin, 
    select_chat_by_number, select_chat_by_name
)
from .group import change_group_description, change_group_name, leave_group
from .get import get_pinned_chats, get_recent_chats, get_group_invite_link
from .login import get_qrcode, login, close

class Whatsapp:

    get_qrcode                  = get_qrcode
    login                       = login
    close                       = close

    add_to_group                = add_to_group
    remove_from_group           = remove_from_group
    make_group_admin            = make_group_admin

    last_message                = last_message
    new_message                 = new_message
    send                        = send
    send_file                   = send_file
    reply                       = reply
    reply_privately             = reply_privately
    reply_file_privately        = reply_file_privately
    select_chat_by_name         = select_chat_by_name
    select_chat_by_number       = select_chat_by_number

    get_pinned_chats            = get_pinned_chats
    get_recent_chats            = get_recent_chats
    get_group_invite_link       = get_group_invite_link

    change_group_description    = change_group_description
    change_group_name           = change_group_name
    leave_group                 = leave_group

    # TODO: Get group info (maybe turn it into a class)

    # TODO: Create group

    # TODO: Change profile picture

    # TODO: Change group profile picture url

whatsapp = Whatsapp()


import os
import re
import shelve
from selenium import webdriver
from os import getlogin, mkdir
from webdriver_manager.chrome import ChromeDriverManager

os.environ['WDM_LOG_LEVEL'] = '0'

def get_qrcode(self):
    """Opens a new chrome page with the QRCode"""

    usr_path = f"C:\\Users\\{getlogin()}\\AppData\\Local\\Google\\Chrome\\User Data\\Default"
    self.mydata = shelve.open('data/data')

    options = webdriver.ChromeOptions()
    options.add_argument("--log-level=OFF")
    options.add_argument(f'--user-data-dir={usr_path}')
    options.add_experimental_option('excludeSwitches', ['enable-logging'])
    driver = webdriver.Chrome(ChromeDriverManager().install(), options=options)

    driver.get('https://web.whatsapp.com')

    self.mydata['user_agent'] = driver.execute_script("return navigator.userAgent;")

    while True:
        try:
            driver.find_element_by_css_selector(
                '#side > div.SgIJV > div > label > div > div._2_1wd.copyable-text.selectable-text')
            break
        except: pass
    driver.close()


def login(self, visible: bool=False):
    """Logs in whatsapp and shows the QRCode if necessary

    Args:
        visible (bool, optional): Shows the process. Defaults to False.
    """

    usr_path = f"C:\\Users\\{getlogin()}\\AppData\\Local\\Google\\Chrome\\User Data\\Default"
    try:
        self.mydata = shelve.open('data/data')
    except:
        mkdir('data')
        self.mydata = shelve.open('data/data')

    try:
        print(f'Logging as: {self.mydata["user_agent"]}')
    except:
        self.get_qrcode()
        
    
    options = webdriver.ChromeOptions()
    options.add_argument(f'--user-data-dir={usr_path}')
    options.add_argument(f"--user-agent={self.mydata['user_agent']}")
    options.add_argument("--start-maximized")
    options.add_argument("--hide-scrollbars")
    options.add_argument("--disable-gpu")
    options.add_argument("--log-level=OFF")
    options.add_experimental_option('excludeSwitches', ['enable-logging'])
    
    if not visible:
        options.add_argument("--headless")

    self.driver = webdriver.Chrome(ChromeDriverManager().install(), options=options)
    self.driver.get('https://web.whatsapp.com')

    while True:
        try:
            if not visible:
                try: 
                    self.driver.find_element_by_css_selector(
                        '#app > div > div > div.landing-window > div.landing-main > div > div.O1rXL > div > canvas')
                    self.driver.close()
                    self.get_qrcode()
                    self.login(visible=visible)
                    break

                except:
                    try:
                        self.driver.find_element_by_xpath(
                            "//a[@title='Atualize o Google Chrome']")
                        self.driver.close()
                        self.get_qrcode()
                        self.login(visible=visible)
                        break
                    
                    except:
                        self.driver.find_element_by_css_selector(
                            '#side > div.SgIJV > div > label > div > div._2_1wd.copyable-text.selectable-text')
                        print('Logged in')
                        break
            else:
                self.driver.find_element_by_css_selector(
                    '#side > div.SgIJV > div > label > div > div._2_1wd.copyable-text.selectable-text')
                print('Logged in')
                break
        except:
            pass


def close(self):
    """Exit the whatsapp"""

    self.driver.close()
    quit()


from setuptools import setup

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='whatsappy-py',
    version='1.9.1',
    description='Whatsappy is a Python library for creating whatsapp bots.',
    packages=['whatsappy'],
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: OS Independent",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires = [
        "selenium ~= 3.141.0",
        "Send2Trash ~= 1.5.0",
        "webdriver-manager ~= 3.2.2",
    ],
    extra_requires = {
        "dev": [
            "pytest>=3.7",
        ]
    },
    url='https://github.com/italoseara/whatsappy',
    author='Italo Seara',
    author_email='italos.seara@gmail.com',
    license='MIT',
)